/*
Esempio NodeMCU con switch Reed e trasmissione
stato via MQTT verso NodeRed.

Questo esempio fa parte della guida di HTML.it "Domotica per Makers"
consultabile all'indirizzo:
https://www.html.it/guide/domotica-per-makers/

L'esempio è realizzato da Giovanni Bernardo (https://www.twitter.com/settorezero)

PREREQUISITI:
- Libreria PubSubClient by Nick O'Leary

*/
 
#include <ESP8266WiFi.h>
#include <PubSubClient.h>

// *** INIZIO SETUP

// IMPOSTAZIONI RETE WI-FI
const char* ssid = "SSID"; // nome della rete Wi-Fi
const char* password = "PASSWORD"; // password della rete Wi-Fi
IPAddress ip(192,168,1,133); // indirizzo IP da assegnare a questo dispositivo
IPAddress gateway(192,168,1,1); // indirizzo IP del tuo router/access-point Wi-Fi
IPAddress subnet(255,255,255,0); // subnet mask

// IMPOSTAZIONI PER LA COMUNICAZIONE MQTT
const char* mqtt_server = "192.168.1.101"; // Indirizzo IP del tuo server MQTT
const char* mqtt_username = "MQTT_USER"; // Nome utente da utilizzare per l'accesso al broker MQTT (metti "" se non hai impostato utente/password)
const char* mqtt_password = "MQTT_PASSWORD"; // Password del tuo server MQTT (metti "" se non hai impostato utente/password)
const uint16_t mqtt_port = 1883; // Porta del tuo server MQTT (default: 1883)
const char* mqtt_client_id = "NodeMCU_Reed"; // Nome univoco a piacere da assegnare a questo dispositivo (due dispositivi sulla rete non possono avere lo stesso nome!)
const char* mqtt_topic = "esempio/finestra"; // Topic sul quale pubblicare il messaggio

// *** FINE SETUP

WiFiClient espClient;
PubSubClient client(mqtt_server, mqtt_port, espClient);

// sul GPIO 2 (D4) del nodeMCU c'è il led blu
#define LED         2 // (D4)
// sul GPIO 5 (D1) abbiamo collegato lo switch reed
#define SWITCH       5 // D1

void mqtt_connect(void) 
  {
  // Eseguiamo un loop continuo fino a che non ci colleghiamo al broker MQTT
  while (!client.connected()) 
    {
    Serial.println("Provo a collegarmi al broker MQTT");
    if (client.connect(mqtt_client_id, mqtt_username, mqtt_password)) 
        {
        Serial.println("Connesso!");
        } 
    else 
        {
        Serial.print("Connessione fallita, codice=");
        Serial.print(client.state());
        Serial.println(" - riprovo tra 5 secondi");
        delay(5000);
        }
    }
  }

void setup() 
  {
  // inizializzo il GPIO dove c'è collegato il led
  pinMode(LED,OUTPUT); // faccio in modo che il GPIO 2, al quale è collegato il led, sia un'uscita
  digitalWrite(LED,HIGH); // led spento. Sul NodeMCU Lolin il led si spegne inviando un livello alto
  // inizializzo il GPIO 5 dove abbiamo collegato lo switch reed
  pinMode (SWITCH,INPUT); // questo pin deve essere un ingresso

  // inizializzo la porta seriale, che userò come debug
  Serial.begin(115200); // baudrate a 115200bps
  
  // inizializzo il Wi-Fi
  Serial.print("Mi sto collegando alla rete Wi-Fi ");
  Serial.println(ssid);

  // Imposto gli IP definiti nella configurazione
  WiFi.config(ip,gateway,subnet);
  // avvio la connessione
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) 
    {
    delay(500);
    Serial.print(".");
    }
  
  Serial.println("");
  Serial.println("Sono connesso al Wi-Fi!");
  digitalWrite(LED,LOW); // accendo il led per segnalare che mi sono collegato

  // Mi collego al broker MQTT
  mqtt_connect();
  }

bool check_switch(void)
  {
  bool published=false;
  // se leggo un livello alto, vuol dire che la finestra è aperta
  if (digitalRead(SWITCH))
    {
    Serial.println("Switch aperto");
    published=client.publish(mqtt_topic,"APERTA");
    }
  else
    // livello basso: finestra chiusa
    {
    Serial.println("Switch chiuso");
    published=client.publish(mqtt_topic,"CHIUSA"); 
    }
  return (published);
  }

void loop() 
  {
  static long lastMsg=millis();

  // ad ogni giro controllo che sono connesso al broker MQTT, in caso contrario
  // rieseguo la connessione
  if (!client.connected()) 
      {
      mqtt_connect();
      }
  client.loop();

  // ogni 2 secondi controllo lo stato della finestra
  if (millis() - lastMsg > 2000) 
    {
    lastMsg = millis();
    if (check_switch())
      {
      Serial.println("Messaggio pubblicato!");
      }
      else
      {
       Serial.println("Errore pubblicazione Messaggio"); 
      }
    }
  }
  
